@@ -36,6 +36,7 @@ ENVIRONMENT::ENVIRONMENT (GLOBALDATA *global)
 	meteors = 0;
 	lightning = 0;
 	fog = 0;
+        wallType = 0.0;
 
 	done = new char[global->screenWidth];
 	fp = new int[global->screenWidth];
@@ -62,9 +63,12 @@ ENVIRONMENT::ENVIRONMENT (GLOBALDATA *global)
 }
 
 // Following are config file definitions
-#define	GFILE_LATEST_VERSION	2
+#define	GFILE_LATEST_VERSION	4
 
+// stop using wind strength as an int. Use the new double version
 #define GFILE_WINDSTRENGTH	&windstrength, sizeof (int), 1, file
+#define GFILE_NEW_WINDSTRENGTH  &windstrength, sizeof (double), 1, file
+ 
 #define	GFILE_WINDVARIATION	&windvariation, sizeof (double), 1, file
 #define	GFILE_VISCOSITY		&viscosity, sizeof (double), 1, file
 #define	GFILE_GRAVITY		&gravity, sizeof (double), 1, file
@@ -74,6 +78,7 @@ ENVIRONMENT::ENVIRONMENT (GLOBALDATA *global)
 #define	GFILE_FOG		&fog, sizeof (double), 1, file
 #define	GFILE_LANDTYPE		&landType, sizeof (double), 1, file
 #define	GFILE_LANDSLIDETYPE	&landSlideType, sizeof (double), 1, file
+#define GFILE_WALLTYPE          &wallType, sizeof(double), 1, file
 
 #define	GFILE_VERSION_1(a)	a (GFILE_WINDSTRENGTH);\
 				a (GFILE_WINDVARIATION);\
@@ -96,9 +101,36 @@ ENVIRONMENT::ENVIRONMENT (GLOBALDATA *global)
 				a (GFILE_LANDTYPE);\
 				a (GFILE_LANDSLIDETYPE);
 
-#define	GFILE_WRITE_LATEST	GFILE_VERSION_2(fwrite)
+#define GFILE_VERSION_3(a)      a (GFILE_WINDSTRENGTH);\
+                                a (GFILE_WINDVARIATION);\
+                                a (GFILE_VISCOSITY);\
+                                a (GFILE_GRAVITY);\
+                                a (GFILE_TECHLEVEL);\
+                                a (GFILE_METEORS);\
+                                a (GFILE_LIGHTNING);\
+                                a (GFILE_FOG);\
+                                a (GFILE_LANDTYPE);\
+                                a (GFILE_LANDSLIDETYPE);\
+                                a (GFILE_WALLTYPE);
+
+#define GFILE_VERSION_4(a)      a (GFILE_NEW_WINDSTRENGTH);\
+                                a (GFILE_WINDVARIATION);\
+                                a (GFILE_VISCOSITY);\
+                                a (GFILE_GRAVITY);\
+                                a (GFILE_TECHLEVEL);\
+                                a (GFILE_METEORS);\
+                                a (GFILE_LIGHTNING);\
+                                a (GFILE_FOG);\
+                                a (GFILE_LANDTYPE);\
+                                a (GFILE_LANDSLIDETYPE);\
+                                a (GFILE_WALLTYPE);
+
+
+#define	GFILE_WRITE_LATEST	GFILE_VERSION_4(fwrite)
 #define	GFILE_READ_VERSION_1	GFILE_VERSION_1(fread)
 #define	GFILE_READ_VERSION_2	GFILE_VERSION_2(fread)
+#define GFILE_READ_VERSION_3    GFILE_VERSION_3(fread)
+#define GFILE_READ_VERSION_4    GFILE_VERSION_4(fread)
 
 int ENVIRONMENT::saveToFile (FILE *file)
 {
@@ -128,6 +160,12 @@ int ENVIRONMENT::loadFromFile (FILE *file)
 		case 2:
 			GFILE_READ_VERSION_2;
 			break;
+                case 3:
+                        GFILE_READ_VERSION_3;
+                        break;
+                case 4:
+                        GFILE_READ_VERSION_4;
+                        break;
 		default:
 			break;
 	}
@@ -249,7 +287,7 @@ int ENVIRONMENT::ingamemenu ()
 	pressed = -1;
 	for (z = 0; z < INGAMEBUTTONS; z++)
 		updatew[z] = 0;
-	show_mouse (NULL);
+	if (! _global->os_mouse) show_mouse (NULL);
 	make_update (_global->halfWidth - 100, _global->halfHeight - 50, 200, 100);
 	rectfill (db, _global->halfWidth - 100, _global->halfHeight - 50, _global->halfWidth + 99, _global->halfHeight + 49, makecol (128, 128, 128));
 	rect (db, _global->halfWidth - 100, _global->halfHeight - 50, _global->halfWidth + 99, _global->halfHeight + 49, BLACK);
@@ -291,12 +329,12 @@ int ENVIRONMENT::ingamemenu ()
 		make_update (lx, ly, ((BITMAP *) (_global->gfxData.M[0].dat))->w, ((BITMAP *) (_global->gfxData.M[0].dat))->h);
 		lx = mouse_x;
 		ly = mouse_y;
-		show_mouse (NULL);
+		if (! _global->os_mouse)  show_mouse (NULL);
 		for (z = 0; z < INGAMEBUTTONS; z++) {
 			draw_sprite (db, (BITMAP *) _global->gfxData.M[(pressed == z) ? 8 : 7].dat, _global->halfWidth - 75, _global->halfHeight + button[z]);
 			textout_centre_ex (db, font, buttext[z], _global->halfWidth, _global->halfHeight + button[z] + 6, WHITE, -1);
 		}
-		show_mouse (db);
+		if (! _global->os_mouse) show_mouse (db);
 		do_updates ();
 	}
 }
