@@ -39,6 +39,15 @@ enum landSlideTypes {
 	LANDSLIDE_GRAVITY,
 };
 
+enum wallTypes {
+        WALL_RUBBER,
+	WALL_STEEL,
+	WALL_SPRING
+};
+
+
+#define SPRING_CHANGE 1.25
+
 class TANK;
 class MISSILE;
 class FLOATTEXT;
@@ -77,6 +86,7 @@ class ENVIRONMENT {
 	int	pclock, mouseclock;
 	int	stage;
 	int	combineUpdates;
+        double  wallType;
 
 	ENVIRONMENT () {};
 	ENVIRONMENT (GLOBALDATA *global);
