@@ -110,3 +110,9 @@ int FLOATTEXT::isSubClass (int classNum)
 	else
 		return (TRUE);
 }
+
+void FLOATTEXT::newRound()
+{
+    age = maxAge + 1;
+}
+
