@@ -24,10 +24,14 @@
 GLOBALDATA::GLOBALDATA ()
 {
 	initialise ();
-        sound = TRUE;
+        sound = 1.0;
+        name_above_tank = TRUE;
 	colourDepth = 16;
-	screenWidth = 800;
-	screenHeight = 600;
+	screenWidth = DEFAULT_SCREEN_WIDTH;
+	screenHeight = DEFAULT_SCREEN_HEIGHT;
+        width_override = height_override = 0;
+        temp_screenWidth = screenWidth;
+        temp_screenHeight = screenHeight;
 	halfWidth = screenWidth / 2;
 	halfHeight = screenHeight / 2;
 	cacheCirclesBG = 1;
@@ -38,6 +42,7 @@ GLOBALDATA::GLOBALDATA ()
 	turntype = TURN_RANDOM;
 	skipComputerPlay = SKIP_HUMANS_DEAD;
 	dataDir = DATA_DIR;
+        os_mouse = 1.0;
 
 	interest = 1.25;
 	scoreHitUnit = 75;
@@ -82,7 +87,7 @@ GLOBALDATA::GLOBALDATA ()
 }
 
 // Following are config file definitions
-#define	GFILE_LATEST_VERSION	4
+#define	GFILE_LATEST_VERSION	6
 
 #define GFILE_NUMPLAYERS	&numPlayers, sizeof (int), 1, file
 #define	GFILE_OLDROUNDS		&rounds, sizeof (int), 1, file
@@ -100,6 +105,11 @@ GLOBALDATA::GLOBALDATA ()
 #define	GFILE_SCOREUNITSELFDESTROY	&scoreUnitSelfDestroy, sizeof (double), 1, file
 #define	GFILE_ACCELERATEDAI	&skipComputerPlay, sizeof (double), 1, file
 #define GFILE_SELLPERCENT	&sellpercent, sizeof(double), 1, file
+#define GFILE_ENABLESOUND       &sound, sizeof(double), 1, file
+#define GFILE_SCREENWIDTH       &screenWidth, sizeof(int), 1, file
+#define GFILE_SCREENHEIGHT      &screenHeight, sizeof(int), 1, file
+#define GFILE_OS_MOUSE          &os_mouse, sizeof(double), 1, file
+
 
 #define	GFILE_VERSION_1(a)	a (GFILE_NUMPLAYERS);\
 				a (GFILE_OLDROUNDS);\
@@ -155,11 +165,54 @@ GLOBALDATA::GLOBALDATA ()
 				a (GFILE_ACCELERATEDAI);\
 				a (GFILE_SELLPERCENT);
 
-#define	GFILE_WRITE_LATEST	GFILE_VERSION_4(fwrite)
+#define GFILE_VERSION_5(a)      a (GFILE_NUMPLAYERS);\
+                                a (GFILE_ROUNDS);\
+                                a (GFILE_DITHER);\
+                                a (GFILE_DETAILEDSKY);\
+                                a (GFILE_DETAILEDLAND);\
+                                a (GFILE_STARTMONEY);\
+                                a (GFILE_TURNTYPE);\
+                                a (GFILE_INTEREST);\
+                                a (GFILE_SCOREROUNDWINBONUS);\
+                                a (GFILE_SCOREHITUNIT);\
+                                a (GFILE_SCORESELFHIT);\
+                                a (GFILE_SCOREUNITDESTROYBONUS);\
+                                a (GFILE_SCOREUNITSELFDESTROY);\
+                                a (GFILE_ACCELERATEDAI);\
+                                a (GFILE_SELLPERCENT);\
+                                a (GFILE_ENABLESOUND);\
+                                a (GFILE_SCREENWIDTH);\
+                                a (GFILE_SCREENHEIGHT);
+
+#define GFILE_VERSION_6(a)      a (GFILE_NUMPLAYERS);\
+                                a (GFILE_ROUNDS);\
+                                a (GFILE_DITHER);\
+                                a (GFILE_DETAILEDSKY);\
+                                a (GFILE_DETAILEDLAND);\
+                                a (GFILE_STARTMONEY);\
+                                a (GFILE_TURNTYPE);\
+                                a (GFILE_INTEREST);\
+                                a (GFILE_SCOREROUNDWINBONUS);\
+                                a (GFILE_SCOREHITUNIT);\
+                                a (GFILE_SCORESELFHIT);\
+                                a (GFILE_SCOREUNITDESTROYBONUS);\
+                                a (GFILE_SCOREUNITSELFDESTROY);\
+                                a (GFILE_ACCELERATEDAI);\
+                                a (GFILE_SELLPERCENT);\
+                                a (GFILE_ENABLESOUND);\
+                                a (GFILE_SCREENWIDTH);\
+                                a (GFILE_SCREENHEIGHT);\
+                                a (GFILE_OS_MOUSE);
+
+
+
+#define	GFILE_WRITE_LATEST	GFILE_VERSION_6(fwrite)
 #define	GFILE_READ_VERSION_1	GFILE_VERSION_1(fread)
 #define	GFILE_READ_VERSION_2	GFILE_VERSION_2(fread)
 #define	GFILE_READ_VERSION_3	GFILE_VERSION_3(fread)
 #define	GFILE_READ_VERSION_4	GFILE_VERSION_4(fread)
+#define GFILE_READ_VERSION_5    GFILE_VERSION_5(fread)
+#define GFILE_READ_VERSION_6    GFILE_VERSION_6(fread)
 
 int GLOBALDATA::saveToFile (FILE *file)
 {
@@ -167,6 +220,9 @@ int GLOBALDATA::saveToFile (FILE *file)
 	if (!file)
 		return (-1);
 
+        screenWidth = (int) temp_screenWidth;
+        screenHeight = (int) temp_screenHeight;
+
 	fwrite (&version, sizeof (int), 1, file);
 
 	GFILE_WRITE_LATEST
@@ -174,9 +230,25 @@ int GLOBALDATA::saveToFile (FILE *file)
 	return (ferror (file));
 }
 
+
+/*
+This function loads the global options from the config file.
+A note on the sound thing going on: Sound is ON by default,
+however, before we get here, Atanks checks the command line
+to see if we should turn sound OFF. So, if sound is OFF we
+we get here, we save that over-ride from the command line.
+However, if the sound is ON when we get here, we leave it
+ON.
+-- Jesse
+*/
 int GLOBALDATA::loadFromFile (FILE *file)
 {
 	int version;
+        double sound_bookmark = 1.0;
+
+        if (! sound)
+            sound_bookmark = sound;
+
 	if (!file)
 		return (-1);
 
@@ -195,10 +267,27 @@ int GLOBALDATA::loadFromFile (FILE *file)
 		case 4:
 			GFILE_READ_VERSION_4;
 			break;
+                case 5:
+                        GFILE_READ_VERSION_5;
+                        break;
+                case 6:
+                        GFILE_READ_VERSION_6;
+                        break;
 		default:
 			break;
 	}
 
+        if (! sound_bookmark)
+           sound = sound_bookmark;
+
+        if (width_override)
+           screenWidth = width_override;
+        if (height_override)
+           screenHeight = height_override;
+
+        halfWidth = screenWidth / 2;
+        halfHeight = screenHeight / 2;
+
 	return (ferror (file));
 }
 
