@@ -22,6 +22,9 @@
 
 #include "main.h"
 
+#define DEFAULT_SCREEN_WIDTH 800
+#define DEFAULT_SCREEN_HEIGHT 600
+
 enum skipComputerPlayType {
 	SKIP_NONE, SKIP_HUMANS_DEAD, SKIP_AUTOPLAY
 };
@@ -59,6 +62,8 @@ class GLOBALDATA {
 	int	colourDepth;
 	int	screenWidth, screenHeight;
 	int	halfWidth, halfHeight;
+        int     width_override, height_override;
+        double  temp_screenWidth, temp_screenHeight;
 	gfxDataStruct	gfxData;
 	DATAFILE	*SOUND;
 
@@ -73,6 +78,7 @@ class GLOBALDATA {
 	double ditherGradients;
 	double detailedLandscape;
 	double detailedSky;
+        double os_mouse;          // whether we should use the OS or custom mouse
 
 	/* All this money data; couldn't it be moved into some separate data
 	structure or object */
@@ -92,7 +98,8 @@ class GLOBALDATA {
 	double	turntype;
 	double	rounds;
 	int     currentround;
-        int     sound;
+        double  sound;
+        int     name_above_tank;
 
 	GLOBALDATA ();
 	void	initialise ();
