@@ -76,7 +76,7 @@ using namespace std;
 #define	BALLISTICS 42
 #define	BEAMWEAPONS 3
 #define WEAPONS (BALLISTICS + BEAMWEAPONS)
-#define ITEMS 17
+#define ITEMS 20
 #define THINGS (WEAPONS + ITEMS)
 #define	NATURALS 6
 #define RADII 6
@@ -94,6 +94,7 @@ using namespace std;
 #define SPREAD 10
 #define SHIELDS 6
 #define WEAPONSOUNDS 4
+#define NAME_LENGTH 24
 
 //Score coeficients
 
@@ -107,9 +108,15 @@ using namespace std;
 #define	MAX_WIND			3
 
 
+
+// bitmaps for tanks
+#define NORMAL_TANK 0
+#define CLASSIC_TANK 1     // really 8 in the .dat file
+
+
 //turns
 enum turnTypes {
-	TURN_HIGH, TURN_LOW, TURN_RANDOM
+	TURN_HIGH, TURN_LOW, TURN_RANDOM, TURN_SIMUL
 };
 
 //options
@@ -136,7 +143,7 @@ class WEAPON
 	public:
 	char		*name;		//name of weapon
 	char		*description;
-	unsigned int cost;		//$ :))
+	int             cost;		//$ :))
 	int		amt;		//number of weapons in one package
 	double		mass;
 	double		drag;
@@ -174,7 +181,7 @@ class ITEM
 	public:
 	char		*name;
 	char		*description;
-	unsigned int		cost;
+	int		cost;
 	int		amt;
 	int		selectable;
 	int		techLevel;
@@ -220,6 +227,8 @@ enum weaponType {
 #define ITEM_NO_SHIELD			-1
 enum itemType {
 	ITEM_TELEPORT,
+        ITEM_SWAPPER,
+        ITEM_FAN,
 	ITEM_VENGEANCE,
 	ITEM_DYING_WRATH,
 	ITEM_FATAL_FURY,
@@ -236,6 +245,7 @@ enum itemType {
 	ITEM_SLICKP,
 	ITEM_DIMPLEP,
 	ITEM_PARACHUTE,
+        ITEM_REPAIRKIT
 };
 
 #define SHIELD_COUNT		6
@@ -301,11 +311,18 @@ int checkPixelsBetweenTwoPoints (GLOBALDATA *global, ENVIRONMENT *env, double *s
 long int calcTotalPotentialDamage (int weapNum);
 long int calcTotalEffectiveDamage (int weapNum);
 
+#ifndef WIN32
+void close_button_handler(void);
+#endif
+
+void doLaunch(GLOBALDATA *gd, ENVIRONMENT *env);
 
 #define GENSKY_DETAILED		1
 #define GENSKY_DITHERGRAD	2
 void generate_sky (GLOBALDATA *global, BITMAP* bmp, const gradient* grad, int flags ) ;
 
+// save all game settings
+int Save_Game_Settings(GLOBALDATA *global, ENVIRONMENT *env, char *path_to_file);
 
 #include "externs.h"
 
