@@ -36,7 +36,7 @@ typedef struct {
 	double	defaultv;
 	char	**specialOpts;
 	char	type;
-	char	viewonly;
+	int	viewonly;
 	int	x;
 	int	y;
 } MENUENTRY;
