@@ -24,6 +24,9 @@
 #include "main.h"
 #include "physobj.h"
 
+#define MAX_MISSLE_AGE 60
+#define MAX_METEOR_AGE 5
+
 class MISSILE: public PHYSICAL_OBJECT
 {
 	public:
@@ -51,3 +54,4 @@ class MISSILE: public PHYSICAL_OBJECT
 };
 
 #endif
+
