@@ -44,6 +44,8 @@ PLAYER::PLAYER (GLOBALDATA *global, ENVIRONMENT *env)
 
 	_targetMatrix = new double[global->screenWidth];
 
+        tank_bitmap = 0.0;
+
 	nm[0] = 99;
 	for (int count = 1; count < WEAPONS; count++)
 		nm[count] = 0;
@@ -51,7 +53,7 @@ PLAYER::PLAYER (GLOBALDATA *global, ENVIRONMENT *env)
 	for (int count = 0; count < ITEMS; count++)
 		ni[count] = 0;
 
-	strncpy (_name, "New", 11);
+	strncpy (_name, "New", NAME_LENGTH);
 	type = HUMAN_PLAYER;
 
 	// 10% of time set to perplay weapon preferences
@@ -73,6 +75,8 @@ PLAYER::PLAYER (GLOBALDATA *global, ENVIRONMENT *env)
 	typeText[5] = "Deadly";
 	preftypeText[0] = "Per Game";
 	preftypeText[1] = "Only Once";
+        tank_type[0] = "Normal";
+        tank_type[1] = "Classic";
 	initMenuDesc ();
 }
 
@@ -85,6 +89,8 @@ PLAYER::PLAYER (GLOBALDATA *global, ENVIRONMENT *env, FILE *file)
 	tank = NULL;
 	_turnStage = 0;
 	selected = FALSE;
+        
+        tank_bitmap = 0.0;
 
 	_targetMatrix = new double[global->screenWidth];
 
@@ -106,6 +112,8 @@ PLAYER::PLAYER (GLOBALDATA *global, ENVIRONMENT *env, FILE *file)
 	typeText[5] = "Deadly";
 	preftypeText[0] = "Per Game";
 	preftypeText[1] = "Only Once";
+        tank_type[0] = "Normal";
+        tank_type[1] = "Classic";
 	initMenuDesc ();
 }
 
@@ -139,6 +147,7 @@ void PLAYER::setComputerValues ()
 			errorMultiplier = 0.1;
 			break;
 		case DEADLY_PLAYER:
+                case PART_TIME_BOT:
 			rangeFindAttempts = 40;
 			retargetAttempts = 10;
 			focusRate = 1.0;
@@ -156,6 +165,7 @@ void PLAYER::setComputerValues ()
 int displayPlayerName (GLOBALDATA *global, ENVIRONMENT *env, int x, int y, void *data);
 void PLAYER::initMenuDesc ()
 {
+
 	int destroyPlayer (GLOBALDATA *global, ENVIRONMENT *env, void *data);
 	menudesc = new MENUDESC;
 	if (!menudesc) {
@@ -165,7 +175,7 @@ void PLAYER::initMenuDesc ()
 	menudesc->title = _name;
 
 	// Name,Color
-	menudesc->numEntries = 7;
+	menudesc->numEntries = 9;
 	menudesc->okayButton = TRUE;
 	menudesc->quitButton = FALSE;
 
@@ -251,15 +261,30 @@ void PLAYER::initMenuDesc ()
 	menuopts[5].x = _global->halfWidth - 3;
 	menuopts[5].y = _global->halfHeight + 32;
 
-	menuopts[6].name = "Delete This Player";
-	menuopts[6].displayFunc = NULL;
-	menuopts[6].color = WHITE;
-	menuopts[6].value = (double*)destroyPlayer;
-	menuopts[6].data = (void*)this;
-	menuopts[6].type = OPTION_ACTIONTYPE;
-	menuopts[6].viewonly = FALSE;
-	menuopts[6].x = _global->halfWidth - 3;
-	menuopts[6].y = _global->halfHeight + 100;
+        menuopts[6].name = "Tank Type";
+        menuopts[6].displayFunc = NULL;
+        menuopts[6].color = WHITE;
+        menuopts[6].value = &tank_bitmap;
+        menuopts[6].min = 0;
+        menuopts[6].max = 1;
+        menuopts[6].increment = 1;
+        menuopts[6].defaultv = 0;
+        menuopts[6].format = "%1.0f";
+        menuopts[6].specialOpts = tank_type;
+        menuopts[6].type = OPTION_SPECIALTYPE;
+        menuopts[6].viewonly = FALSE;
+        menuopts[6].x = _global->halfWidth - 3;
+        menuopts[6].y = _global->halfHeight + 52;
+
+	menuopts[8].name = "Delete This Player";
+	menuopts[8].displayFunc = NULL;
+	menuopts[8].color = WHITE;
+	menuopts[8].value = (double*)destroyPlayer;
+	menuopts[8].data = (void*)this;
+	menuopts[8].type = OPTION_ACTIONTYPE;
+	menuopts[8].viewonly = FALSE;
+	menuopts[8].x = _global->halfWidth - 3;
+	menuopts[8].y = _global->halfHeight + 82;
 
 	menudesc->entries = menuopts;
 }
@@ -274,9 +299,10 @@ PLAYER::~PLAYER ()
 }
 
 // Following are config file definitions
-#define	GFILE_LATEST_VERSION	9
+#define	GFILE_LATEST_VERSION	11
 
 #define GFILE_NAME		_name, sizeof (char), 11, file
+#define GFILE_NAME_EXTENDED     _name, sizeof (char), NAME_LENGTH, file
 #define	GFILE_WEAPONPREFERENCE1	_weaponPreference, sizeof (int), 51, file
 #define	GFILE_WEAPONPREFERENCE2	_weaponPreference, sizeof (int), 54, file
 #define	GFILE_WEAPONPREFERENCE3	_weaponPreference, sizeof (int), 58, file
@@ -293,6 +319,8 @@ PLAYER::~PLAYER ()
 #define	GFILE_SELFPRESERVATION	&selfPreservation, sizeof (double), 1, file
 #define	GFILE_PAINSENSITIVITY	&painSensitivity, sizeof (double), 1, file
 #define	GFILE_DEFENSIVE		&defensive, sizeof (double), 1, file
+#define GFILE_TANK_BITMAP       &tank_bitmap, sizeof (double), 1, file
+#define GFILE_TURRET_BITMAP    &turret_bitmap, sizeof (double), 1, file
 
 #define	GFILE_VERSION_1(a)	a (GFILE_NAME);\
 				a (GFILE_TYPE);\
@@ -386,7 +414,39 @@ PLAYER::~PLAYER ()
 				a (GFILE_PAINSENSITIVITY);\
 				a (GFILE_DEFENSIVE);
 
-#define	GFILE_WRITE_LATEST	GFILE_VERSION_9(fwrite)
+#define GFILE_VERSION_10(a)     a (GFILE_NAME_EXTENDED);\
+                                a (GFILE_VENGEFUL);\
+                                a (GFILE_VENGEANCETHRESHOLD);\
+                                a (GFILE_TYPE);\
+                                a (GFILE_COLOR);\
+                                a (GFILE_COLOR2);\
+                                a (GFILE_WEAPONPREFERENCE5);\
+                                a (GFILE_PLAYED);\
+                                a (GFILE_WON);\
+                                a (GFILE_PREFTYPE);\
+                                a (GFILE_SELFPRESERVATION);\
+                                a (GFILE_PAINSENSITIVITY);\
+                                a (GFILE_DEFENSIVE);
+
+#define GFILE_VERSION_11(a)     a (GFILE_NAME_EXTENDED);\
+                                a (GFILE_VENGEFUL);\
+                                a (GFILE_VENGEANCETHRESHOLD);\
+                                a (GFILE_TYPE);\
+                                a (GFILE_COLOR);\
+                                a (GFILE_COLOR2);\
+                                a (GFILE_WEAPONPREFERENCE5);\
+                                a (GFILE_PLAYED);\
+                                a (GFILE_WON);\
+                                a (GFILE_PREFTYPE);\
+                                a (GFILE_SELFPRESERVATION);\
+                                a (GFILE_PAINSENSITIVITY);\
+                                a (GFILE_DEFENSIVE);\
+                                a (GFILE_TANK_BITMAP);\
+                                a (GFILE_TURRET_BITMAP); // remove this line in version 12
+
+
+
+#define	GFILE_WRITE_LATEST	GFILE_VERSION_11(fwrite)
 #define	GFILE_READ_VERSION_1	GFILE_VERSION_1(fread)
 #define	GFILE_READ_VERSION_2	GFILE_VERSION_2(fread)
 #define	GFILE_READ_VERSION_3	GFILE_VERSION_3(fread)
@@ -396,13 +456,22 @@ PLAYER::~PLAYER ()
 #define GFILE_READ_VERSION_7	GFILE_VERSION_7(fread)
 #define GFILE_READ_VERSION_8	GFILE_VERSION_8(fread)
 #define GFILE_READ_VERSION_9	GFILE_VERSION_9(fread)
+#define GFILE_READ_VERSION_10   GFILE_VERSION_10(fread);
+#define GFILE_READ_VERSION_11   GFILE_VERSION_11(fread);
+
 
 int PLAYER::saveToFile (FILE *file)
 {
 	int version = GFILE_LATEST_VERSION;
+        double turret_bitmap;  // quick hack to fix version 11
+
 	if (!file)
 		return (-1);
 
+        // make sure part time bots are saved as human
+        if (type == PART_TIME_BOT)
+           type = HUMAN_PLAYER;
+
 	fwrite (&version, sizeof (int), 1, file);
 
 	GFILE_WRITE_LATEST
@@ -413,6 +482,8 @@ int PLAYER::saveToFile (FILE *file)
 int PLAYER::loadFromFile (FILE *file)
 {
 	int version;
+        double turret_bitmap;     // quick hack to fix version 11
+
 	if (!file)
 		return (-1);
 
@@ -446,6 +517,12 @@ int PLAYER::loadFromFile (FILE *file)
 		case 9:
 			GFILE_READ_VERSION_9;
 			break;
+                case 10:
+                        GFILE_READ_VERSION_10;
+                        break;
+                case 11:
+                        GFILE_READ_VERSION_11;
+                        break;
 		default:
 			break;
 	}
@@ -470,6 +547,10 @@ int PLAYER::loadFromFile (FILE *file)
 		defensive = (double)((rand () % 10000) - 5000) / 5000.0;
 	}
 
+        // make sure previous human players are restored as humans
+        if (type == PART_TIME_BOT)
+           type = HUMAN_PLAYER;
+
 	return (ferror (file));
 }
 
@@ -484,8 +565,14 @@ void PLAYER::exitShop ()
 		damageMultiplier += pow (tmpDM, 0.6);
 }
 
+
+// run this at the begining of each turn
 void PLAYER::newRound ()
 {
+        // if the player is under computer control, give it back to the player
+        if ( type == PART_TIME_BOT)
+           type = HUMAN_PLAYER;
+
 	setComputerValues ();
 	if (!tank) {
 		tank = new TANK(_global, _env);
@@ -496,6 +583,17 @@ void PLAYER::newRound ()
 		tank->player = this;
 	}
 	tank->newRound ();
+
+        /* I am not thrilled about putting this here, but we need a
+        way to avoid having too much money or too little (haha) money.
+        -- Jesse
+        */
+        if (money > 1000000000)
+           money = 1000000000;
+        if (money < 0)
+           money = 0;
+
+        changed_weapon = false;
 }
 
 void PLAYER::initialise ()
@@ -547,7 +645,9 @@ int PLAYER::selectRandomItem ()
 
 void PLAYER::setName (char *name)
 {
-	strncpy (_name, name, 10);
+        // initalize name
+        memset(_name, '\0', NAME_LENGTH);
+	strncpy (_name, name, NAME_LENGTH - 1);
 }
 
 int PLAYER::controlTank (TANK *ctank)
@@ -557,7 +657,8 @@ int PLAYER::controlTank (TANK *ctank)
 
 	if (key_shifts & KB_CTRL_FLAG && ctrlUsedUp) {
 		if (key[KEY_LEFT] || key[KEY_RIGHT] ||
-			key[KEY_UP] || key[KEY_DOWN])
+			key[KEY_UP] || key[KEY_DOWN] ||
+                        key[KEY_PGUP] || key[KEY_PGDN])
 			ctrlUsedUp = TRUE;
 		else
 			ctrlUsedUp = FALSE;
@@ -584,7 +685,8 @@ int PLAYER::controlTank (TANK *ctank)
 			void clockadd ();
 			install_int_ex (clockadd, SECS_TO_TIMER(6000));
 			int mm = _env->ingamemenu ();
-			install_int_ex (clockadd, BPS_TO_TIMER (100));
+			// install_int_ex (clockadd, BPS_TO_TIMER (100));
+                        install_int_ex(clockadd, BPS_TO_TIMER(FRAMES_PER_SECOND));
 			_env->make_update (0, 0, _global->screenWidth, _global->screenHeight);
 			_env->make_bgupdate (0, 0, _global->screenWidth, _global->screenHeight);
 			
@@ -742,9 +844,21 @@ int PLAYER::traceShellTrajectory (TANK *ctank, int *estimateX, int *estimateY, d
 
 		startX = x;
 		startY = y;
+                // hit side of the screen
 		if (x + xv < 1 || x + xv > (_global->screenWidth-1)) {
-			xv = -xv;	//bounce on the border
-			direction = -direction;
+                      // bounce if wall is rubber
+                      if ( _env->wallType == WALL_RUBBER )
+                      {
+		  	  xv = -xv;	//bounce on the border
+			  direction = -direction;
+                      }
+                      else if ( _env->wallType == WALL_SPRING )
+                      {
+                          xv = -xv * SPRING_CHANGE;
+                          direction = -direction; 
+                      }
+                      else   // wall is steel
+                          hitSomething = 1;
 		} else {
 			// motion - wind affected
 			double accel = (_env->wind - xv) / weapon[ctank->cw].mass * drag * _env->viscosity;
@@ -1005,7 +1119,7 @@ int PLAYER::addTankDamageToMatrix (TANK *ctank, TANK *ltank, int weapNum)
 	for (int cx = 0; cx < _global->screenWidth; cx++) {
 		// Weapon specific calculations...
 		int damage = 0;
-		double distance;
+		double distance = 0.0;   // just to avoid compiler warnings
 		int cy = _env->surface[cx];
 		int effectiveTankX = (int)(tankX + ((tankY - cy) * windAffect));
 		if (weapNum >= RIOT_CHARGE && weapNum <= RIOT_BLAST) {
@@ -1138,8 +1252,8 @@ double PLAYER::selectTarget (int *targetXCoord, int *targetYCoord)
  */
 double PLAYER::calcDefenseValue (TANK *ctank, TANK *ltank)
 {
-	double strengthRatio;
-	double armourRatio;
+	// double strengthRatio;
+	double armourRatio = 1.0;    // set a default, any default
 
 	if (ctank->l > 0 && ltank->l > 0) {
 		if (ctank->l + ctank->sh <= ltank->l + ltank->sh) {
@@ -1151,9 +1265,9 @@ double PLAYER::calcDefenseValue (TANK *ctank, TANK *ltank)
 		}
 	}
 
-	strengthRatio = armourRatio;
-
-	return (strengthRatio);
+	// strengthRatio = armourRatio;
+	// return (strengthRatio);
+        return armourRatio;
 }
 
 int PLAYER::computerSelectItem (TANK *ctank)
@@ -1204,7 +1318,7 @@ int PLAYER::computerSelectItem (TANK *ctank)
 					}
 				}
 			} else {
-				double score;
+				double score = 0.0;   // to avoid warnings
 				int itemNum = cw - WEAPONS;
 
 				if ((!ni[itemNum]) ||
@@ -1257,7 +1371,7 @@ int PLAYER::computerControls (TANK *ctank)
 
 	} else if (_turnStage == CALCULATE_ATTACK) {
 		int overshoot;
-		int lastOvershoot;
+		int lastOvershoot = 0;    // avoid compiler warning
 		int estimateX, estimateY;
 		int xdist = (int)(_targetX - ctank->x);
 		int ydist = (int)(_targetY - ctank->y);
@@ -1343,7 +1457,9 @@ int PLAYER::computerControls (TANK *ctank)
 	} else if (fi) {
 		// if (fi) don't do any of the following
 	} else if (_turnStage == FIRE_WEAPON) {
-		ctank->activateCurrentSelection ();
+		// ctank->activateCurrentSelection ();
+                ctank->simActivateCurrentSelection();
+                gloating = false;
 		_turnStage = 0;
 	}
 	return (0);
@@ -1402,6 +1518,24 @@ int PLAYER::humanControls (TANK *ctank)
 				if (key_shifts & KB_CTRL_FLAG)
 					ctrlUsedUp = TRUE;
 			}
+                        if ( key[KEY_PGUP] && !ctrlUsedUp && ctank->p < MAX_POWER )
+                        {
+                            ctank->p += 100;
+                            if (ctank->p > MAX_POWER)
+                               ctank->p = MAX_POWER;
+                            _global->updateMenu = 1;
+			    if (key_shifts & KB_CTRL_FLAG)
+				ctrlUsedUp = TRUE;
+                        }
+                        if ( key[KEY_PGDN] && !ctrlUsedUp && ctank->p > 0)
+                        {
+                            ctank->p -= 100;
+                            if (ctank->p < 0)
+                               ctank->p = 0;
+                            _global->updateMenu = 1;
+		            if (key_shifts & KB_CTRL_FLAG)
+			      ctrlUsedUp = TRUE;
+                        }
 		}
 	}
 	if (k && !fi) {
@@ -1428,11 +1562,41 @@ int PLAYER::humanControls (TANK *ctank)
 					}
 					//calcDamageMatrix (ctank, ctank->cw);
 					//selectTarget ();
+                                        changed_weapon = false;
 				}
+                                if ( k >> 8 == KEY_BACKSPACE) {
+                                        _global->updateMenu = 1;
+                                        while (1) {
+                                                ctank->cw--;
+                                                if (ctank->cw < 0)
+                                                   ctank->cw = THINGS - 1;
+ 
+                                                if (ctank->cw < WEAPONS) {
+                                                        if (ctank->player->nm[ctank->cw])
+                                                                break;
+                                                } else {
+                                                        if (item[ctank->cw - WEAPONS].selectable && ctank->player->ni[ctank->cw - WEAPONS])
+                                                                break;
+
+                                                }
+                                        }
+                                        changed_weapon = false;
+                                }
+
+                                // put the tank under computer control
+                                if ((k >> 8 == KEY_F10) )
+                                {
+                                    type = PART_TIME_BOT;
+                                    setComputerValues();
+                                    return (computerControls (ctank));
+                                }
+
 				if ((k >> 8 == KEY_SPACE) &&
 					(((ctank->cw < WEAPONS) && (ctank->player->nm[ctank->cw] > 0)) ||
 						((ctank->cw < THINGS) && (ctank->player->ni[ctank->cw - WEAPONS] > 0)))) {
-					ctank->activateCurrentSelection ();
+				//	ctank->activateCurrentSelection ();
+                                ctank->simActivateCurrentSelection();
+                                gloating = false;
 				}
 			}
 		}
