@@ -21,6 +21,7 @@
 #include "main.h"
 #include "menu.h"
 
+
 enum playerType {
 	HUMAN_PLAYER,
 	USELESS_PLAYER,
@@ -29,6 +30,7 @@ enum playerType {
 	TARGETTER_PLAYER,
 	DEADLY_PLAYER,
 	LAST_PLAYER_TYPE,
+        PART_TIME_BOT         // normally a human, but acting as a deadly computer
 };
 //player weapon preference type
 //ALWAYS_PREF - only choose weapon preferences once on player creation
@@ -51,7 +53,7 @@ enum turnStages {
 class TANK;
 class PLAYER {
 	private:
-	char	_name[11];
+	char	_name[NAME_LENGTH];
 	int	_currTank;
 	GLOBALDATA *_global;
 	ENVIRONMENT *_env;
@@ -69,7 +71,8 @@ class PLAYER {
 	public:
 	double	type;
 	double preftype;
-	char* preftypeText[ALWAYS_PREF+1];
+	char* preftypeText[ALWAYS_PREF + 1];
+        char *tank_type[2];
 	PLAYER	*revenge;
 	int	vengeful;	// 0-100 chance of retaliation
 	double	vengeanceThreshold;	// Damage required to warrant revenge
@@ -85,7 +88,7 @@ class PLAYER {
 	int	score;
 	double	played, won;
 	double	selected;
-	unsigned long int	money;
+	int	money;
 	double	damageMultiplier;
 	TANK	*tank;
 	int	color;
@@ -95,6 +98,8 @@ class PLAYER {
 	MENUDESC	*menudesc;
 	char *typeText[LAST_PLAYER_TYPE];
         bool changed_weapon;
+        bool gloating;
+        double tank_bitmap;      // which type of tank do we have
 
 	PLAYER (GLOBALDATA *global, ENVIRONMENT *env);
 	PLAYER (GLOBALDATA *global, ENVIRONMENT *env, FILE *file);
