@@ -77,6 +77,8 @@ TANK::~TANK ()
 	_env->removeObject (this);
 	delete shieldText;
 	delete healthText;
+        if (nameText)
+          delete nameText;
 }
 
 TANK::TANK (GLOBALDATA *global, ENVIRONMENT *env)
@@ -88,12 +90,25 @@ TANK::TANK (GLOBALDATA *global, ENVIRONMENT *env)
 		perror ("tank.cc: Failed allocating memory for healthText in TANK::TANK");
 		exit (1);
 	}
-	shieldText = new FLOATTEXT (global, env, "", 0, 0,
-			makecol (200, 200, 255), CENTRE);
+
+        shieldText = new FLOATTEXT (global, env, "", 0, 0, makecol (200, 200, 255), CENTRE);
 	if (!shieldText) {
 		perror ("tank.cc: Failed allocating memory for shieldText in TANK::TANK");
 		exit (1);
 	}
+
+        if (global->name_above_tank)
+        {
+            nameText = new FLOATTEXT (global, env, "", 0, 0, WHITE, CENTRE);
+            if (! nameText)
+            {
+                perror ("tank.cc: Failed allocating memory for nameText in TANK::TANK");
+                exit(1);
+            }
+        }
+        else
+           nameText = NULL;
+
 	initialise ();
 
 	_align = LEFT;
@@ -136,18 +151,28 @@ void TANK::newRound ()
 	shPhase = rand () % 360;
 	sht = 0;
 	l = 100;
+        repair_rate = 0;
 	if (player) {
 		double tmpL = 0;
 		tmpL += (int)(player->ni[ITEM_ARMOUR] * item[ITEM_ARMOUR].vals[0]);
 		tmpL += (int)(player->ni[ITEM_PLASTEEL] * item[ITEM_PLASTEEL].vals[0]);
+                repair_rate = Get_Repair_Rate();
 		if (tmpL > 0)
 			l += (int)pow (tmpL, 0.6);
+
+                healthText->set_color( player->color );
+                if (nameText)
+                {
+                   nameText->set_text ( player->getName() );
+                   nameText->set_color ( player->color );
+                }
 	}
 	maxLife = l;
 	ds = 0;
 	fs = 0;
 	sprintf (buf, "%d", l);
 	healthText->set_text (buf);
+
 }
 
 void TANK::update ()
@@ -165,14 +190,37 @@ void TANK::applyDamage ()
 	char buf[10];
 	FLOATTEXT *damageText;
 	FLOATTEXT *revengeText;
+        FLOATTEXT *gloatText;
 
 	if (damage > sh + l)
 		damage = sh + l;
 	sh -= (int)damage;
 	if (creditTo) {
 		if (player != creditTo) {	//enemy hit ++
-			creditTo->money += (unsigned long int)(damage * _global->scoreHitUnit);
-			if ((int)player->type != HUMAN_PLAYER) {
+			creditTo->money += (int)(damage * _global->scoreHitUnit);
+                        // this tank is destroyed, the attacker gloats
+                        if ( (damage >= sh + l) && (! creditTo->gloating) )
+                        {
+                           // avoid trying to print victory message over a dead tank
+                           if (creditTo->tank)
+                           {
+                                gloatText = new FLOATTEXT (_global, _env,
+                                        creditTo->selectGloatPhrase( (double) damage / maxLife),
+                                        (int) creditTo->tank->x, (int) creditTo->tank->y - 30,
+                                        creditTo->color, CENTRE);
+                                if (! gloatText)
+                                {
+                                   perror ("tank.cc: Failed allocating memory for gloatTExt in applyDamage.");
+                                   exit(1);
+                                }
+                                gloatText->xv = 0;
+                                gloatText->yv = -0.4;
+                                gloatText->maxAge = 300;
+                                creditTo->gloating = true;
+                            }
+                        }
+
+                        if ((int)player->type != HUMAN_PLAYER) {
 				if (player->revenge == creditTo) {
 					player->annoyanceFactor += damage;
 				} else {
@@ -185,7 +233,7 @@ void TANK::applyDamage ()
 						(int)x, (int)y - 30,
 						player->color, CENTRE);
 					if (!revengeText) {
-						perror ("tank.cc: Failed allocating memory for revengeText in TANK::TANK");
+						perror ("tank.cc: Failed allocating memory for revengeText in applyDamage");
 						exit (1);
 					}
 					revengeText->xv = 0;
@@ -197,7 +245,7 @@ void TANK::applyDamage ()
                         if ( (creditTo->money - (damage * _global->scoreSelfHit)) < 0)
                            creditTo->money = 0;
                         else
-			   creditTo->money -= (unsigned long int)(damage * _global->scoreSelfHit);
+			   creditTo->money -= (int)(damage * _global->scoreSelfHit);
 			// if (creditTo->money < 0)
 			//	creditTo->money = 0;
 		}
@@ -371,18 +419,44 @@ void TANK::printHealth (BITMAP *dest, int offset)
 	if (sh > 0)
 		textpos -= 10;
 	healthText->set_pos ((int)x, (int)y - TANKHEIGHT + textpos + offset);
+
+        // display player name
+        if (nameText)
+        {
+           textpos -= 10;
+           nameText->set_pos ((int)x, (int)y - TANKHEIGHT + textpos + offset);
+        }
 }
 
 void TANK::draw (BITMAP *dest, int healthOffset)
 {
 	int turretAngle;
 
+        // get bitmap for tank
+        if (player)
+        {
+            switch ( (int) this->player->tank_bitmap)
+            {
+               case CLASSIC_TANK:
+                       use_tank_bitmap = 8;
+                       use_turret_bitmap = 1;
+                       turret_x = x;
+                       turret_y = y + (TANKHEIGHT / 2);
+                       break;
+               default: use_tank_bitmap = 0;
+                        use_turret_bitmap = 0;
+                        turret_x = x;
+                        turret_y = y;
+                         break;
+            }
+        }
+
 	rectfill (dest, (int) x - (TANKWIDTH - 1),
 		(int) (y + TANKHEIGHT) - 2,
 		(int) (x + TANKWIDTH) - 2,
 		(int) (y + TANKHEIGHT),
 		this->player->color);
-	draw_sprite (dest, (BITMAP *) _global->gfxData.T[0].dat, (int) x - TANKWIDTH, (int) y);
+	draw_sprite (dest, (BITMAP *) _global->gfxData.T[use_tank_bitmap].dat, (int) x - TANKWIDTH, (int) y);
 	if (sh > 0) {
 		int phaseValue = 1;
 
@@ -403,8 +477,8 @@ void TANK::draw (BITMAP *dest, int healthOffset)
 		drawing_mode(DRAW_MODE_SOLID, NULL, 0, 0);
 	}
 	turretAngle = (int) ((float) (90 - a) * ((float) 256 / (float) 360));
-	rotate_sprite (dest, (BITMAP *) _global->gfxData.TG[0].dat,
-		       (int) x - GUNLENGTH, (int) y - (GUNLENGTH - 2), itofix (turretAngle));
+	rotate_sprite (dest, (BITMAP *) _global->gfxData.TG[use_turret_bitmap].dat,
+		       (int) turret_x - GUNLENGTH, (int) turret_y - (GUNLENGTH - 2), itofix (turretAngle));
 
 	if (sh > 0)
 		setUpdateArea ((int)x - TANKWIDTH - 15, (int)y - TANKHEIGHT,
@@ -447,10 +521,43 @@ int TANK::get_heaviest_shield ()
 	return ITEM_NO_SHIELD;
 }
 
+
+
+
+void TANK::simActivateCurrentSelection ()
+{
+       char buf[16];
+
+       if (_global->turntype != TURN_SIMUL)
+       {
+               activateCurrentSelection();
+       } else {
+               _env->stage = 1;
+       }
+
+       // apply repairs
+       l += repair_rate;
+       if (l > maxLife)
+          l = maxLife;
+       sprintf(buf, "%d", l);
+       healthText->set_text(buf);
+
+       // avoid having key presses read in next turn
+       clear_keybuf();
+}
+
+
+
+
 void TANK::activateCurrentSelection ()
 {
 	int z;
 
+        // avoid firing weapons on exit in Widnows
+        if ( (_global->command == GLOBAL_COMMAND_QUIT) || 
+             (_global->command == GLOBAL_COMMAND_MENU) )
+           return;
+
 	if (cw < WEAPONS) {
                 player->changed_weapon = false;
 		if (cw)
@@ -471,8 +578,8 @@ void TANK::activateCurrentSelection ()
 				myv = _global->slope[ca][1] * p * (100.0 / FRAMES_PER_SECOND) / 100;
 
 				newmis = new MISSILE(_global, _env,
-					x + (_global->slope[ca][0] * GUNLENGTH) /*- mxv*/,
-					y + (_global->slope[ca][1] * GUNLENGTH) /*- myv*/,
+					turret_x + (_global->slope[ca][0] * GUNLENGTH) /*- mxv*/,
+					turret_y + (_global->slope[ca][1] * GUNLENGTH) /*- myv*/,
 					mxv, myv, cw);
 				if (!newmis) {
 					perror ("tank.cc: Failed allocating memory for newmis in TANK::activateCurrentSelection");
@@ -489,7 +596,7 @@ void TANK::activateCurrentSelection ()
 					newmis->drag *= item[ITEM_SLICKP].vals[0];
 				}
 			}
-		} else { // BEAM weapon
+		} else {        // BEAM weapon
 			play_sample ((SAMPLE *) _global->SOUND[weapon[cw].sound].dat, 255, 128, 1000, 0);
 			for (z = 0; z < _env->am; z++) {
 				BEAM *newbeam;
@@ -498,8 +605,8 @@ void TANK::activateCurrentSelection ()
 				ca = a + ((SPREAD * z) - (SPREAD * (_env->am - 1) / 2));
 
 				newbeam = new BEAM (_global, _env,
-					x + (_global->slope[ca][0] * GUNLENGTH),
-					y + (_global->slope[ca][1] * GUNLENGTH),
+					turret_x + (_global->slope[ca][0] * GUNLENGTH),
+					turret_y + (_global->slope[ca][1] * GUNLENGTH),
 					ca, cw);
 				if (!newbeam) {
 					perror ("tank.cc: Failed allocating memory for newbeam in TANK::activateCurrentSelection");
@@ -511,7 +618,7 @@ void TANK::activateCurrentSelection ()
 			}
 
 		}
-	} else {
+	} else {         // activate an item
 		int itemNum = cw - WEAPONS;
 		if (itemNum < ITEM_VENGEANCE || itemNum > ITEM_FATAL_FURY)
 			player->ni[itemNum]--;
@@ -526,7 +633,46 @@ void TANK::activateCurrentSelection ()
 				perror ("tank.cc: Failed allocating memory for teleport in TANK::activateCurrentSelection");
 				exit (1);
 			}
-		} else if ((itemNum >= ITEM_VENGEANCE) &&
+                } else if (itemNum == ITEM_SWAPPER) {
+                     int random_tank_number;
+                     TANK *other_tank;
+                     TELEPORT *my_teleport, *other_teleport;
+
+                     // pick a random tank (not us)
+                     random_tank_number = rand() % _global->numTanks;
+                     other_tank = _env->order[random_tank_number];
+                     while ( (! other_tank) || (other_tank == this) )
+                     {
+                        random_tank_number++;
+                        if (random_tank_number > _global->maxNumTanks)
+                           random_tank_number = 0;
+                        other_tank = _env->order[random_tank_number];
+                     }
+                     creditTo = player;
+                     // create a teleport ojbect for this tank
+                     my_teleport = new TELEPORT (_global, _env, this, (int) other_tank->x, (int) other_tank->y, TANKHEIGHT * 4 + GUNLENGTH, 120);
+                     // create a teleport object for the other tank
+                     other_teleport = new TELEPORT (_global, _env, other_tank, (int) x, (int) y, TANKHEIGHT * 4 + GUNLENGTH, 120);            
+                
+		}
+                else if ( itemNum == ITEM_FAN )
+                {
+                    // play wind sound
+		    play_sample ((SAMPLE *) _global->SOUND[2].dat, 255, 128, 1000, 0);
+                    if (a < 180)   // move wind to the right
+                       _env->wind += (p / 20);
+                    else     // wind to the left
+                       _env->wind -= (p / 20);
+
+                    // make sure wind is not too strong
+                    if (_env->wind < (-_env->windstrength / 2) )
+                        _env->wind = -_env->windstrength / 2;
+                    else if (_env->wind > (_env->windstrength / 2) )
+                        _env->wind = _env->windstrength / 2;
+
+                    _env->lastwind = _env->wind;
+
+                } else if ((itemNum >= ITEM_VENGEANCE) &&
 			(itemNum <= ITEM_FATAL_FURY)) {
 			creditTo = player;
 			damage = l + sh;
@@ -647,3 +793,30 @@ int TANK::tank_on_tank( GLOBALDATA *global )
    return found_tank;  
 }
 
+
+
+/*
+This function figures out how many points a tank
+will repair itself each turn. This is based
+on the number of repair kits a player has.
+The amount is returned as an int.
+-- Jesse
+*/
+int TANK::Get_Repair_Rate()
+{
+     int num_kits;
+     int repair_units = 0;
+     int increase_amount = 5;
+
+     num_kits = player->ni[ITEM_REPAIRKIT];
+     while (num_kits > 0)
+     {
+         repair_units += increase_amount;
+         if (increase_amount > 1)
+            increase_amount--;         
+         num_kits--;
+     }
+
+     return repair_units;
+}
+
