@@ -25,6 +25,7 @@
 
 #define	TELETIME	120
 
+
 class PLAYER;
 class EXPLOSION;
 class FLOATTEXT;
@@ -41,6 +42,7 @@ class TANK: public PHYSICAL_OBJECT
 	int 	t, a, p, cw, l, sh, sht, fs, ds, para, pen;
 	int	maxLife;	// amount awarded at beginning of round
 	double	damage;
+        int     repair_rate;
 	int	repulsion;
 	int	shieldColor, shieldThickness;
 	int	flashdamage, hit;
@@ -50,6 +52,9 @@ class TANK: public PHYSICAL_OBJECT
 	double	shPhase;
 	PLAYER	*creditTo;
 	FLOATTEXT	*healthText, *shieldText;//, *damageText;
+        FLOATTEXT       *nameText;
+        int use_tank_bitmap, use_turret_bitmap;
+        double turret_x, turret_y;
 
 	TANK ();
 	~TANK ();
@@ -63,6 +68,7 @@ class TANK: public PHYSICAL_OBJECT
 	void	update ();
 	void	requireUpdate ();
 	void	explode ();
+        void    simActivateCurrentSelection();
 	void	activateCurrentSelection ();
 	int	isSubClass (int classNum);
 	int	getClass () {return (TANK_CLASS);};
@@ -74,6 +80,7 @@ class TANK: public PHYSICAL_OBJECT
 	void	newRound ();
 	void	framelyAccounting ();
         int     tank_on_tank(GLOBALDATA *gd); //is this tank on top of another?
+        int     Get_Repair_Rate();    // how many units a tank will repair itself per round
 };
 
 #endif
